@@ -1255,6 +1255,8 @@ KateIconBorder::KateIconBorder ( KateViewInternal* internalView, QWidget *parent
   , m_viewInternal( internalView )
   , m_iconBorderOn( false )
   , m_lineNumbersOn( false )
+  , m_viRelLineNumbersOn( false )
+  , m_updateViRelLineNumbers ( false )
   , m_foldingMarkersOn( false )
   , m_dynWrapIndicatorsOn( false )
   , m_annotationBorderOn( false )
@@ -1315,9 +1317,9 @@ void KateIconBorder::setAnnotationBorderOn( bool enable )
 void KateIconBorder::removeAnnotationHovering()
 {
   // remove hovering if it's still there
-  if (m_annotationBorderOn && !m_hoveredAnnotationText.isEmpty())
+  if (m_annotationBorderOn && !m_hoveredAnnotationGroupIdentifier.isEmpty())
   {
-    m_hoveredAnnotationText.clear();
+    m_hoveredAnnotationGroupIdentifier.clear();
     hideAnnotationTooltip();
     QTimer::singleShot( 0, this, SLOT(update()) );
   }
@@ -1336,6 +1338,29 @@ void KateIconBorder::setLineNumbersOn( bool enable )
   QTimer::singleShot( 0, this, SLOT(update()) );
 }
 
+void KateIconBorder::setViRelLineNumbersOn(bool enable)
+{
+  if( enable == m_viRelLineNumbersOn )
+    return;
+
+  m_viRelLineNumbersOn = enable;
+  /*
+   * We don't have to touch the m_dynWrapIndicatorsOn because
+   * we already got it right from the m_lineNumbersOn
+   */
+  updateGeometry();
+
+  QTimer::singleShot( 0, this, SLOT(update()) );
+}
+
+void KateIconBorder::updateViRelLineNumbers()
+{
+  if (m_viRelLineNumbersOn) {
+    m_updateViRelLineNumbers = true;
+    update();
+  }
+}
+
 void KateIconBorder::setDynWrapIndicators( int state )
 {
   if (state == m_dynWrapIndicators )
@@ -1507,6 +1532,7 @@ void KateIconBorder::paintBorder (int /*x*/, int y, int /*width*/, int height)
   uint startz = (y / h);
   uint endz = startz + 1 + (height / h);
   uint lineRangesSize = m_viewInternal->cache()->viewCacheLineCount();
+  uint currentLine = m_view->cursorPosition().line();
 
   // center the folding boxes
   int m_px = (h - 11) / 2;
@@ -1625,7 +1651,8 @@ void KateIconBorder::paintBorder (int /*x*/, int y, int /*width*/, int height)
         }
 
         // Draw a border around all adjacent entries that have the same text as the currently hovered one
-        if( m_hoveredAnnotationText == text.toString() )
+        const QVariant identifier = model->data( realLine, (Qt::ItemDataRole) KTextEditor::AnnotationModel::GroupIdentifierRole );
+        if( m_hoveredAnnotationGroupIdentifier == identifier.toString() )
         {
           p.drawLine( lnX, y, lnX, y+h );
           p.drawLine( lnX+borderWidth, y, lnX+borderWidth, y+h );
@@ -1675,7 +1702,20 @@ void KateIconBorder::paintBorder (int /*x*/, int y, int /*width*/, int height)
 
       if (realLine > -1) {
         if (m_viewInternal->cache()->viewLine(z).startCol() == 0) {
-          if (m_lineNumbersOn) {
+          if (m_viRelLineNumbersOn && m_view->viInputMode()) {
+            int diff = abs(realLine - currentLine);
+            if (diff > 0) {
+              p.drawText( lnX + m_maxCharWidth / 2, y, lnWidth - m_maxCharWidth, h,
+                          Qt::TextDontClip|Qt::AlignRight|Qt::AlignVCenter, QString("%1").arg(diff) );
+            } else {
+              p.drawText( lnX + m_maxCharWidth / 2, y, lnWidth - m_maxCharWidth, h,
+                          Qt::TextDontClip|Qt::AlignLeft|Qt::AlignVCenter, QString("%1").arg(realLine + 1) );
+            }
+            if (m_updateViRelLineNumbers) {
+              m_updateViRelLineNumbers = false;
+              update();
+            }
+          } else if (m_lineNumbersOn) {
             p.drawText( lnX + m_maxCharWidth / 2, y, lnWidth - m_maxCharWidth, h,
                         Qt::TextDontClip|Qt::AlignRight|Qt::AlignVCenter, QString("%1").arg( realLine + 1 ) );
           }
@@ -1926,7 +1966,8 @@ void KateIconBorder::mouseMoveEvent( QMouseEvent* e )
         m_view->annotationModel() : m_doc->annotationModel();
       if (model)
       {
-        m_hoveredAnnotationText = model->data( t.line(), Qt::DisplayRole ).toString();
+        m_hoveredAnnotationGroupIdentifier = model->data( t.line(),
+                                                          (Qt::ItemDataRole) KTextEditor::AnnotationModel::GroupIdentifierRole ).toString();
         showAnnotationTooltip( t.line(), e->globalPos() );
         QTimer::singleShot( 0, this, SLOT(update()) );
       }
@@ -1936,7 +1977,7 @@ void KateIconBorder::mouseMoveEvent( QMouseEvent* e )
       if( positionToArea( e->pos() ) == IconBorder )
         m_doc->requestMarkTooltip( t.line(), e->globalPos() );
 
-      m_hoveredAnnotationText.clear();
+      m_hoveredAnnotationGroupIdentifier.clear();
       hideAnnotationTooltip();
       QTimer::singleShot( 0, this, SLOT(update()) );
     }
